#!/bin/bash

# -------------------------------------------------------------------
#
#　        radiko タイムフリー録音スクリプト
#
# -------------------------------------------------------------------
VER='1.2.3'


#
# システム変数 （変更不要）
#
ver_mes="=====  rec_radiko.sh ver ${VER}  ====="
pid=$$
uname=`whoami`
hname='rdk'
scpath=$0
tdate='00000000'
hour='000000'
stop='000000'
term=0
brdate=0
fndate=0
qmode=1
savdir='.'
pname=''
vol=256		# 音量（256:音量維持）
retryc=3	# サーバ接続の retry 回数
intval=20	# retry 待機秒数＆増加幅　※retryごとに増えます。例）20→40→60→80…
cnvtmp=0	# 1:encodeもテンポラリ領域で行う（容量に注意）


#
# radiko用変数 （必要に応じて設定）
#
app="pc_html5"
version="0.0.1"
user="dummy_user"
device="pc"
radiko_auth_key="bcd151073c03b352e1ef2fd66c32209da9ca0afa"
radiko_login_uri='https://radiko.jp/ap/member/webapi/member/login'
radiko_logout_uri='https://radiko.jp/v4/api/member/logout'
mail=''		# ラジコプレミアム
pass=''		# ラジコプレミアム


#
# 録音モードと音声品質 （必要に応じて設定）
#
rmode='mp3'	# 'mp3' or 'm4a' or 'raw'
smprate=22050	#  sampling rate
bitrate='32k'	#  bit rate
ac=2		#  2:stero   1:mono


#
# テンポラリディレクトリ指定 （必要に応じて設定）
#
wkdir='/tmp'


#
# Timezone
#
#export TZ="Asia/Tokyo"



#
# general function
#
print_err() {
  echo [`date '+%m/%d %H:%M:%S'`] "[ERROR][$pname] $1" >&2
}
print_echo() {
  echo [`date '+%H:%M:%S'`] $1
}
bin_check() {
   which $1 1>/dev/null
   if [ $? != 0 ]; then
      echo "[ERROR] command '$1' is not found. please install it." >&2; exit
   fi
}


#
# コマンドの存否チェック
#
bin_check ffmpeg
bin_check curl



#
# コマンドラインから引数を取得
#
if [ $# -eq 1 ]; then
   pname=$1
   tdate=`date '+%Y%m%d'`
elif [ $# -eq 2 ]; then
   pname=$1
   tdate=$2
   if [[ $tdate =~ ^[0-9]{1,2}$ ]]; then
      tdate=`date -d "$tdate days ago" '+%Y%m%d'`
   fi
elif [ $# -eq 4 ]; then
   channel=$1
   tdate=$2
   hour=$3
   stop=$4
   pname="__MANUAL__"
   if [[ $tdate =~ ^[0-9]{1,2}$ ]]; then
      tdate=`date -d "$tdate days ago" '+%Y%m%d'`
   fi
   fname="$channel $tdate $hour"
else
   echo ""
   echo "$ver_mes"
   echo "  usage: rec_radiko.sh [options]" >&2
   echo "options: program_name | program_name targetdate | channel targetdate hour stop"
   echo ""
   exit 1
fi




#
# incファイルの取得（わかる人向け。.sampleファイル参照）
#
incf=${scpath/.sh/.inc}		# rec_radiko.sh → rec_radiko.inc 名前変換
if [ -r $incf ]; then
   source $incf
fi



#
# 番組用パラメータをセット 【各自で設定】
#
if [ $channel ]; then
  :
elif [ $pname = 'haya' ]; then
  channel='TBS'
  savdir="/home/${uname}/radiko/haya"
  fname="林原めぐみのTokyo Boogie Night !date!"
  hour='000000'
  stop='003000'
  term=1738
  fndate=1
elif [ $pname = 'baka' ]; then
  channel='TBS'
  savdir="/home/${uname}/radiko/baka"
  fname="深夜の馬鹿力 !date!"
  hour='010000'
  stop='030000'
  fndate=1
elif [ $pname = "test" ]; then
  channel='RN2'
  fname="test_rec"
  hour='085900'
  stop='090000'
  brdate=$(date +"%u")
#  rmode='raw'
elif [ $pname = 'inctest' ]; then
  echo "[ERROR] include file not found. exit." >&2
  exit 1
else
  echo "[ERROR] bad program name --> '$pname'. Exit!" >&2
  exit 1
fi



#
# simple check [ directory ]
#
if [ ! -w $wkdir ]; then
  echo "[ERROR][$pname] wkdir is not exist or not writable." >&2
  exit 1
fi
if [ ! -w $savdir ]; then
  echo "[ERROR][$pname] savdir is not exist or not writable." >&2
  exit 1
fi


#
# simple check [ hour & stop ]
#
if [[ ! $hour =~ ^([0-9]{6})$ ]]; then
  echo "[ERROR][$pname] 'hour' is invalid. ($hour)" >&2
  exit 1
fi
if [[ ! $stop =~ ^([0-9]{6})$ ]]; then
  echo "[ERROR][$pname] 'stop' is invalid. ($stop)" >&2
  exit 1
fi
if [ $hour -ge $stop ]; then
  echo "[ERROR][$pname] Invalid 'hour' or 'stop'." >&2
  exit 1
fi


#
# simple check [rmode]
#
if [ $rmode != 'mp3' ] && [ $rmode != 'm4a' ] && [ $rmode != 'raw' ]; then
  echo "[ERROR][$pname] 'rmode' is invalid. ($rmode)" >&2
  exit 1
fi



#
# brdate,fndateによる日付調整
#
recdate=`date -d "$tdate $brdate days ago" '+%Y%m%d'`
fndate0=`date -d "$tdate $fndate days ago" '+%Y年%m月%d日'`

if [[ ! $recdate =~ ^([0-9]{8})$ ]]; then
  echo "[ERROR] targetdate is invalid. ($tdate)" >&2
  exit 1
fi


#
# 指定された録音日時が録音できる範囲内か否かの簡易チェック
#
frday=`date -d "8 days ago" '+%Y%m%d'`
today=`date -d "0 days ago" '+%Y%m%d'`
ntime=`date -d "0 days ago" '+%H%M00'`

if [ $recdate -lt $frday ] || [ $recdate -gt $today ]; then
   echo "[ERROR][$pname] 'targetdate' is invalid. ($tdate $recdate)" >&2
   exit 1
fi

if [ $recdate -eq $today ] && [ $hour -ge $ntime ]; then
  echo "[ERROR][$pname] 'targetdate''hour' too early. ($recdate $hour)" >&2
  date
  exit 1
fi




#
#  ver_mes
#
if [ $qmode -eq 1 ]; then print_echo "$ver_mes"; fi


#
# set filename
#
fname=${fname/!date!/$fndate0}


#
# set qmode
#
if [ $qmode -eq 1 ]; then
   fquiet='-loglevel fatal'
   print_echo 'quiet mode（進捗状況を詳細表示しません）'
   echo "[ simple info >> [$pname][$wkdir][${savdir}/$fname][$rmode]"
else
   fquiet=''
fi



#
#  set cookie name & path
#
cookie="${wkdir}/${hname}_${uname}_${channel}_${pname}_cookie.txt"



#
# エリアフリー関数（ラジコプレミアム）
#
radiko_login_status=0
radiko_login() {
   local code=$(curl -s -c "$cookie" -o /dev/null -X POST -w '%{http_code}' \
     --data-urlencode "mail=${mail}" --data-urlencode "pass=${pass}" "$radiko_login_uri")

   if [ $code -ne 200 ] ; then
      return
   fi
   radiko_login_status=1
   print_echo "+++ radiko premium mode +++"
}
radiko_logout() {
   curl -s -b "$cookie" -o /dev/null -X POST "$radiko_logout_uri"
   print_echo "+++ radiko premium mode closed +++ "
   radiko_login_status=0
   rm -f "$cookie"
}
term_stop() {
   if [ $radiko_login_status -eq 1 ]; then radiko_logout; fi
   print_echo "rec_radiko terminated."
   exit 1
}
trap "term_stop" HUP INT



# ----------------------------------------------
#
#            radiko ログイン
#
# ----------------------------------------------
if [ $mail ] && [ $pass ] ; then
   MAX_RETRY=$retryc
   INTERVAL=$intval
   for i in $(seq 1 $MAX_RETRY); do

      radiko_login

      if [ $radiko_login_status -eq 1 ]; then
          break
      fi

      print_err "$i 回目(loop at radiko_login)"
      sleep $INTERVAL
      INTERVAL=$(( INTERVAL + intval ))
   done

   if [ $radiko_login_status -eq 0 ]; then
      print_err "radiko_login error. exit. (Invalid E-mail or Password. or login server down?)"
      exit 1
   fi
fi



#
# Access Token 取得
#
MAX_RETRY=$retryc
INTERVAL=$intval
for i in $(seq 1 $MAX_RETRY); do

   # ----- auth1
   auth1_res=$(curl \
       --silent -b "$cookie" \
       --header "X-Radiko-App: ${app}" \
       --header "X-Radiko-App-Version: ${version}" \
       --header "X-Radiko-Device: ${device}" \
       --header "X-Radiko-User: ${user}" \
       --dump-header - \
       --output /dev/null \
       "https://radiko.jp/v2/api/auth1")

   # +++ Get authtoken, keyoffset and keylength
   authtoken=$(echo "${auth1_res}" | \
           awk 'tolower($0) ~/^x-radiko-authtoken: / {print substr($0,21,length($0)-21)}')
   keyoffset=$(echo "${auth1_res}" | \
           awk 'tolower($0) ~/^x-radiko-keyoffset: / {print substr($0,21,length($0)-21)}')
   keylength=$(echo "${auth1_res}" | \
           awk 'tolower($0) ~/^x-radiko-keylength: / {print substr($0,21,length($0)-21)}')

   if [ -z "${authtoken}" ] || [ -z "${keyoffset}" ] || [ -z "${keylength}" ]; then
      print_err "$i 回目(loop) ($authtoken,$keylength,$keyoffset)"
      sleep $INTERVAL
      INTERVAL=$(( INTERVAL + intval ))
        # echo "$auth1_res" > /tmp/auth1_res.txt
      continue
   fi

   # +++ Make partial key
   partialkey=$(echo "${radiko_auth_key}" | \
                  dd bs=1 "skip=${keyoffset}" "count=${keylength}" 2> /dev/null | base64)


   # ----- auth2
   code=$(curl \
       --silent -b "$cookie" -w '%{http_code}' \
       --header "X-Radiko-AuthToken: ${authtoken}" \
       --header "X-Radiko-PartialKey: ${partialkey}" \
       --header "X-Radiko-Device: ${device}" \
       --header "X-Radiko-User: ${user}" \
       --output /dev/null \
       "https://radiko.jp/v2/api/auth2")

   if [ $code -eq 200 ]; then
       break
   fi

   print_err "$i 回目(loop at auth2)"
   sleep $INTERVAL
   INTERVAL=$(( INTERVAL + intval ))
done

if [ $code -ne 200 ] ; then
   print_err "Invalid AutoToken. (code is $code)"
   if [ $radiko_login_status -eq 1 ]; then radiko_logout; fi
   exit 1
fi



#
# ----- Recording start -----
#
if [ $qmode -eq 1 ]; then
   print_echo 'AutoToken success. Recording start.'
fi
echo "  AuthToken : ${authtoken}"
echo "  KeyLehgth : ${keylength}"
echo "  KeyOffset : ${keyoffset}"
echo "  PartialKey: ${partialkey}"
echo "  HTTP code : ${code}"

#
# set url
#
ft="${recdate}$hour"
to="${recdate}$stop"

if [ $stop -gt 239999 ] ; then
    to=`date --date "${recdate} 1 days" +%Y%m%d`"$(printf "%06d" $((stop - 240000)))"
fi


recurl="https://radiko.jp/v2/api/ts/playlist.m3u8?station_id=${channel}&l=15&ft=${ft}&to=${to}"


#if [ $qmode -eq 1 ]; then
   echo "  cnvtmp    : $cnvtmp"
   echo "  Channel   : ${channel}"
   echo "  Date      : ${recdate}"
   echo "  From      : ${ft}"
   echo "  To        : ${to}"
   echo "  Filename  : ${fname}"
   echo "  URL       : ${recurl}"
#fi





#
# 録音ファイル名（raw）
#
rectmpfile="${wkdir}/${hname}_${uname}_${channel}_${pname}.m4a"


#rm -f "$rectmpfile"


#
# ダウンロード
#
MAX_RETRY=$retryc
INTERVAL=$intval
for i in $(seq 1 $MAX_RETRY); do
   #echo "$i 回目"
   ffmpeg -headers "X-Radiko-AuthToken: ${authtoken}" -y -i "$recurl" $fquiet \
             -bsf:a aac_adtstoasc -acodec copy -vn "$rectmpfile" && break

   print_err "$i 回目(loop at ffmpeg)"
   sleep $INTERVAL && /bin/false
   INTERVAL=$(( INTERVAL + intval ))
done


if [ $? -ne 0 ] || [ ! -f "$rectmpfile" ]; then
   print_err "ffmpeg process failed. (server down? or bad channel name?)"
   if [ $radiko_login_status -eq 1 ]; then radiko_logout; fi
   exit 1
fi



#
# エリアフリー（ログアウト）
#
if [ $radiko_login_status -eq 1 ]; then radiko_logout; fi


if [ $qmode -eq 1 ]; then
   print_echo 'Recording completed. start encoding...'
fi



# ----------------------------------------------
#
#　　　        　エンコード
#
# ----------------------------------------------

#
# ffmpeg encode option
#
opt1="-ar $smprate -ab $bitrate -ac $ac -vol $vol"
opt2="-ss 0 -t $term"
opt3="$fquiet"
ext="m4a"


case $rmode in
   mp3) codec="libmp3lame"; ext="mp3" ;;
   m4a) codec="aac -strict experimental" ;;
     *) rmode='raw' ;;
esac



#
# term option
#
if [ $term -eq 0 ]; then
   opt2=''
fi

#
# 最終出力ファイル名とテンポラリ出力ファイル名
#
recoutfile="${savdir}/${fname}.$ext"
cnvtmpfile="${wkdir}/${hname}_${uname}_${channel}_${pname}_tmp.$ext"

rm -f "$recoutfile"
rm -f "$cnvtmpfile"


if [ $rmode = 'raw' ]; then

   ffmpeg -y -i "$rectmpfile" -acodec copy $opt2 $opt3 "$recoutfile"

elif [ $cnvtmp -eq 1 ]; then
   ffmpeg -y -i "$rectmpfile" -acodec $codec $opt1 $opt2 $opt3 "$cnvtmpfile"
   mv "$cnvtmpfile" "$recoutfile"

else
   ffmpeg -y -i "$rectmpfile" -acodec $codec $opt1 $opt2 $opt3 "$recoutfile"
fi


rm -f "$rectmpfile"


if [ $qmode -eq 1 ]; then
   print_echo 'Encoding completed. program exit.'
fi


if [ $pname = 'test' ] || [ $pname = 'test2' ]; then
   print_echo '--- this is TEST MODE --- （テストモード終了）'
fi



#
#  ver_mes
#
if [ $qmode -eq 0 ]; then print_echo "$ver_mes"; fi



exit


#-------------------------------------------------------------
# 【配布場所】
#
# http://mycc.s33.xrea.com/data/rpi/rec_radiko.html
#
#-------------------------------------------------------------
